
import cloudproc
from cloudproc.config import DATASET_DIR, VGG_WEIGHTS_FILE, TRAIN_FILE
from cloudproc.cloud_cnn import _parse_function, heights_to_labels, num_height_classes
from cloudproc.models import ConvnetVGG16
import tensorflow as tf
import os

vgg_weight_file = os.path.join(DATASET_DIR, VGG_WEIGHTS_FILE)
train_file = os.path.join(DATASET_DIR, TRAIN_FILE)


train_dataset = tf.data.TFRecordDataset(train_file).map(_parse_function)
train_dataset = train_dataset.batch(5)
training_iterator = train_dataset.make_one_shot_iterator()

images_big, height_readings, rain_quantity, sun_insolation = training_iterator.get_next()
labels = heights_to_labels(height_readings)
# resize for vgg16 (224x224)
images = tf.image.resize_images(images_big, (224,224))


# Build the graph for the conv net
conv_net = ConvnetVGG16(images, classes=num_height_classes())
#height_preds = conv_net.height_predictions()
height_class_preds = conv_net.class_predictions()
keep_prob = conv_net.keep_prob

# height bin classification + infinity height classification
classification_loss = conv_net.cross_entropy_loss(labels)
loss = tf.reduce_mean(classification_loss)
conv_accuracy = conv_net.accuracy(labels)

with tf.name_scope('adam_optimizer'):
    global_step = tf.Variable(0, name='global_step', trainable=False)
    train_step = tf.train.AdamOptimizer(1e-3).minimize(loss, global_step=global_step)

init_op = tf.group(tf.global_variables_initializer())

with tf.Session() as sess:
    sess.run(init_op)
    conv_net.load_weights(vgg_weight_file, sess)
    try:
        #loaded = sess.run(conv_net.load_vars)
        #for n, arr in zip(loaded, map(lambda x: x.name, conv_net.load_vars)):
        #        print(n, arr)
        for i in range(20):

            _, tloss, closs, heights, h_classes, h_class_preds = sess.run([train_step, loss,
                classification_loss, height_readings, labels, height_class_preds], feed_dict={keep_prob:0.5})
            print('Loss:', tloss, closs)
            print(heights)
            print(h_classes, '=?=', h_class_preds)

    except tf.errors.OutOfRangeError as e:
        print('done')
